/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __TIZEN_CAPI_NETWORK_USER_AWARENESS_H__
#define __TIZEN_CAPI_NETWORK_USER_AWARENESS_H__

#include <stdbool.h>
#include <glib.h>
#include <tizen_error.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file user-awareness.h
 */

/**
 * @ingroup CAPI_NETWORK_FRAMEWORK
 * @addtogroup CAPI_NETWORK_UA_MODULE User Awareness
 * @{
 */

#ifndef TIZEN_ERROR_UA
#define TIZEN_ERROR_UA 0x11170000 /**< TODO: This should move to tizen_error.h */
#endif

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Max. MAC address length.
 * @since_tizen 5.5
 */
#define UA_MAC_ADDRESS_STRING_LEN 18

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Max. IPv4 address length.
 * @since_tizen 5.5
 */
#define UA_IPV4_ADDRESS_STRING_LEN 16

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Max. IP address length (e.g. IPv6).
 * @since_tizen 5.5
 */
#define UA_IP_ADDRESS_MAX_STRING_LEN 50

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Max. device ID length (e.g. UUID).
 * @since_tizen 5.5
 */
#define UA_MOBILE_ID_STRING_LEN 50

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Max. user name length.
 * @since_tizen 5.5
 */
#define UA_USER_NAME_MAX_STRING_LEN 50

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Max. user account length.
 * @since_tizen 5.5
 */
#define UA_USER_ACCOUNT_MAX_STRING_LEN 70

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Device's BLE iBeacon advertisement length.
 * @since_tizen 5.5
 */
#define UA_IBEACON_ADV_MAX_LEN 31

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Device's BLE payload DUID length.
 * @since_tizen 5.5
 */
#define UA_BLE_PAYLOAD_DUID_LEN 17

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Device's BT mac address(payload) length.
 * @since_tizen 5.5
 */
#define UA_BT_MAC_MAX_LEN 18

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Device scan time multiplier.
 * @since_tizen 5.5
 */
#define UA_SCAN_TIME_MULTIPLIER 10

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Max. service name length.
 * @since_tizen 5.5
 */
#define UA_MAX_SERVICE_LEN 50

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Window step multiplier.
 * @since_tizen 5.5
 */
#define UA_DETECTION_WINDOW_STEP 5  /* 5 seconds */

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Detection cycle multiplier.
 * @since_tizen 5.5
 */
#define UA_DETECTION_CYCLE_STEP	60  /* 1 min */

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Enumerations of UA framework error codes.
 * @since_tizen 5.5
 */
typedef enum {
	UA_ERROR_NONE = TIZEN_ERROR_NONE, /**< Successful*/
	UA_ERROR_NOT_PERMITTED = TIZEN_ERROR_NOT_PERMITTED, /**< Operation not permitted */
	UA_ERROR_INVALID_PARAMETER = TIZEN_ERROR_INVALID_PARAMETER, /**< Invalid parameter */
	UA_ERROR_OUT_OF_MEMORY = TIZEN_ERROR_OUT_OF_MEMORY, /**< Out of memory */
	UA_ERROR_PERMISSION_DENIED = TIZEN_ERROR_PERMISSION_DENIED, /**< Permission denied */
	UA_ERROR_RESOURCE_BUSY = TIZEN_ERROR_RESOURCE_BUSY, /**< Device or resource busy */
	UA_ERROR_TIMED_OUT = TIZEN_ERROR_TIMED_OUT, /**< Timeout error */
	UA_ERROR_CANCELED = TIZEN_ERROR_CANCELED, /**< Operation canceled */
	UA_ERROR_NOW_IN_PROGRESS = TIZEN_ERROR_NOW_IN_PROGRESS, /**< Operation now in progress */
	UA_ERROR_NOT_SUPPORTED = TIZEN_ERROR_NOT_SUPPORTED, /**< Not Supported */
	UA_ERROR_NO_DATA = TIZEN_ERROR_NO_DATA, /**< No data available */
	UA_ERROR_NOT_INITIALIZED = TIZEN_ERROR_UA | 0x01, /**< Not initialized */
	UA_ERROR_NOT_IN_PROGRESS = TIZEN_ERROR_UA | 0x02, /**< Operation Not in progress */
	UA_ERROR_ALREADY_DONE = TIZEN_ERROR_UA | 0x03, /**< Operation already done */
	UA_ERROR_OPERATION_FAILED = TIZEN_ERROR_UA | 0x04, /**< Operation Failed */
	UA_ERROR_NOT_READY = TIZEN_ERROR_UA | 0x05, /**< Resource not ready */
	UA_ERROR_NOT_ENABLED = TIZEN_ERROR_UA | 0x06, /**< Not enabled */
	UA_ERROR_NOT_FOUND = TIZEN_ERROR_UA | 0x10, /**< Not found */
	UA_ERROR_ALREADY_REGISTERED = TIZEN_ERROR_UA | 0x11, /**< Already registered */
	UA_ERROR_DB_FAILED = TIZEN_ERROR_UA | 0x12, /**< DB operation faield */
	UA_ERROR_NOT_REGISTERED = TIZEN_ERROR_UA | 0x13, /**< Not registered */
} ua_error_code_e;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Enumerations of UA framework supported sensors.
 * @since_tizen 5.5
 */
typedef enum {
	UA_SENSOR_BT = 0x0001, /**< BT Sensor */
	UA_SENSOR_BLE = 0x0002, /**< BLE Sensor */
	UA_SENSOR_WIFI = 0x0004, /**< Wi-Fi Sensor */
	UA_SENSOR_MOTION = 0x0008, /**< Motion Sensor */
	UA_SENSOR_LIGHT = 0x0010, /**< Light Sensor */
	UA_SENSOR_AUDIO = 0x0020, /**< Audio Sensor */
	UA_SENSOR_MAX /**< Sensor Max. */
} ua_sensor_e;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Enumerations of UA framework user detection modes.
 * @since_tizen 5.5
 */
typedef enum {
	UA_DETECT_MODE_ALL_SENSOR = 0x01, /**< Detection mode all */
	UA_DETECT_MODE_ANY_SENSOR, /**< Detection mode any */
	UA_DETECT_MODE_INVALID /**< <Invalid detection mode */
} ua_detection_mode_e;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Enumerations of User device Operating Systems.
 * @since_tizen 5.5
 */
typedef enum {
	UA_OS_TYPE_NOT_DEFINE = 0x00, /**< OS not defined */
	UA_OS_TYPE_TIZEN, /**< Tizen */
	UA_OS_TYPE_ANDROID, /**< Android */
	UA_OS_TYPE_IOS, /**< iOS */
} ua_os_type_e;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Enumerations of User device mac address types.
 * @since_tizen 5.5
 */
typedef enum {
	UA_MAC_TYPE_NONE = 0x00, /**< None */
	UA_MAC_TYPE_BT = 0x01, /**< BT MAC */
	UA_MAC_TYPE_BLE = 0x02, /**< BLE MAC */
	UA_MAC_TYPE_WIFI = 0x04, /**< Wi-Fi station mode MAC */
	UA_MAC_TYPE_P2P = 0x08, /**< Wi-Fi P2P mode MAC */
	UA_MAC_TYPE_INVALID /**< <Invalid MAC type */
} ua_mac_type_e;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Enumerations of device presence state.
 * @since_tizen 5.5
 */
typedef enum {
	UA_PRESENCE_STATE_INACTIVATE = 0x00, /**< User state is inactive */
	UA_PRESENCE_STATE_ACTIVATE, /**< User state is active */
	UA_PRSENCE_STATE_INVALID = 0xFF /**< User state invalid */
} ua_presence_state_e;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Enumerations of active scan type.
 * @since_tizen 5.5
 */
typedef enum {
	UA_ACTIVE_SCAN_TYPE_DEVICE_FOUND = 0x00, /**< Device found */
	UA_ACTIVE_SCAN_TYPE_COMPLETED = 0x01 /**< SCAN Completed */
} ua_active_scan_type_e;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Enumerations of logical conjunction operation in between
 *         AND and OR user detection conditions.
 * @since_tizen 5.5
 */
typedef enum {
	UA_OR_OPERATION = 0x00, /**< AND | OR */
	UA_AND_OPERATION = 0x01 /**< AND & OR */
} ua_condition_conjunction_e;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief The handle of sensor information.
 * @since_tizen 5.5
 */
typedef void *ua_sensor_h;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief The handle of user presence/absence monitor.
 * @since_tizen 5.5
 */
typedef void *ua_monitor_h;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief The handle of user information.
 * @since_tizen 5.5
 */
typedef void *ua_user_h;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief The handle of service information.
 * @since_tizen 5.5
 */
typedef void *ua_service_h;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief The handle of device information.
 * @since_tizen 5.5
 */
typedef void *ua_device_h;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to be invoked if a sensor state changed (ready <-> not ready).
 * @since_tizen 5.5
 *
 * @remarks The @a handle should not be released.
 * @remarks The @a handle can be used only in the callback.
 *
 * @param[in] ready TRUE is sensor is ready FALSE otherwise.
 * @param[in] sensor Sensor type for which state was changed.
 * @param[in] handle The monitor handle for which sensor type was added.
 * @param[in] user_data The user data passed in ua_monitor_set_sensor_state_cb().
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_set_sensor_state_cb()
 */
typedef void (*ua_monitor_sensor_state_changed_cb)(
		bool ready,
		ua_sensor_e sensor,
		ua_monitor_h handle,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to get all sensors for a handle.
 * @since_tizen 5.5
 *
 * @param[in] sensor Sensor type which is present in UA monitor.
 * @param[in] user_data User data passed in ua_monitor_foreach_sensor().
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_foreach_sensor()
 */
typedef bool (*ua_monitor_foreach_sensor_cb)(
		ua_sensor_e sensor,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to be invoked on completion scan for each device one by one.
 * @since_tizen 5.5
 *
 * @remarks The @a handle should not be released.
 * @remarks The @a handle can be used only in the callback.
 * @remarks The @a device_handle should not be released.
 * @remarks The @a device_handle can be used only in the callback.
 *
 * @param[in] type To indicate if active scan type is set to device being found or scan complete.
 * @param[in] handle The monitor handle
 * @param[in] device_handle The device handle to detected device's information. When scan
 *  is complete, it is set to NULL.
 * @param[in] user_data The user data passed in ua_monitor_start_scan_devices()
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_start_scan_devices()
 */
typedef void (*ua_scan_completed_cb)(
		ua_active_scan_type_e type,
		ua_monitor_h handle,
		ua_device_h device_handle,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to be invoked on detection of user presence.
 * @since_tizen 5.5
 *
 * @remarks The @a handle should not be released.
 * @remarks The @a handle can be used only in the callback.
 * @remarks The @a sensor_handle should not be released.
 * @remarks The @a sensor_handle can be used only in the callback.
 *
 * @param[in] result The result of the requested operation.
 * @param[in] handle The monitor handle
 * @param[in] sensor Sensor type, In case monitor has more than one sensor and detection
 *  mode is not #UA_DETECT_MODE_ANY_SENSOR, sensor will be set to last sensor which
 *  reported user presence before invoking callback.
 * @param[in] device_handle The first device handle on which sensor is detected.
 * @param[in] sensor_info The sensor info for the detected sensor.
 * @param[in] user_data The user data passed in ua_monitor_start_presence_detection()
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_start_presence_detection()
 */
typedef void (*ua_presence_detected_cb)(
		int result,
		ua_monitor_h handle,
		ua_sensor_e sensor,
		ua_device_h device_handle,
		ua_sensor_h sensor_handle,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to be invoked on detection of user absence.
 * @since_tizen 5.5
 *
 * @remarks The @a handle should not be released.
 * @remarks The @a handle can be used only in the callback.
 * @remarks The @a sensor_handle should not be released.
 * @remarks The @a sensor_handle can be used only in the callback.
 *
 * @param[in] result The result of the requested operation.
 * @param[in] handle The monitor handle with which absence detection start was invoked. \n
 * @param[in] sensor Sensor type by which absence was detected. \n
 *  In case monitor has more than one sensor and detection mode is not
 *  #UA_DETECT_MODE_ANY_SENSOR, sensor will be set to #UA_SENSOR_MAX before
 *  invoking callback.
 * @param[in] sensor_handle The sensor handle for the detected sensor.
 * @param[in] user_data The user data passed in ua_monitor_start_absence_detection()
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_start_absence_detection()
 */
typedef void (*ua_absence_detected_cb)(
		int result,
		ua_monitor_h handle,
		ua_sensor_e sensor,
		ua_sensor_h sensor_handle,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to be invoked on absence detection for one user.
 * @since_tizen 5.5
 *
 * @remarks The @a handle should not be released.
 * @remarks The @a handle can be used only in the callback.
 * @remarks The @a user_handle should not be released.
 * @remarks The @a user_handle can be used only in the callback.
 * @remarks The @a sensor_handles should not be released.
 * @remarks The @a sensor_handles can be used only in the callback.
 *
 * @param[in] result The result of the requested operation.
 * @param[in] handle The monitor handle with which absence detection start was invoked.
 * @param[in] user_handle The user handle to detected user's information.
 *   In case monitor has more than one sensor and detection mode is not
 *   #UA_DETECT_MODE_ANY_SENSOR, sensor will be set to #UA_SENSOR_MAX before invoking
 *   callback.
 * @param[in] sensor_handles The list of sensor handles on which user is not detected.
 * @param[in] user_data  The user data passed in ua_monitor_set_user_absence_detected_cb()
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_set_user_absence_detected_cb()
 */
typedef void (*ua_absence_user_detected_cb)(
		int result,
		ua_monitor_h handle,
		ua_user_h user_handle,
		GSList *sensor_handles,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to be invoked on presence detection for one user.
 * @since_tizen 5.5
 *
 * @remarks The @a handle should not be released.
 * @remarks The @a handle can be used only in the callback.
 * @remarks The @a user_handle should not be released.
 * @remarks The @a user_handle can be used only in the callback.
 * @remarks The @a device_handles should not be released.
 * @remarks The @a device_handles can be used only in the callback.
 * @remarks The @a sensor_handles should not be released.
 * @remarks The @a sensor_handles can be used only in the callback.
 *
 * @param[in] result The result of the requested operation.
 * @param[in] handle The monitor handle with which presence detection start was invoked.
 * @param[in] user_handle The user handle to detected user's information.
 * @param[in] device_handles The list of device handles on which user is detected.
 * @param[in] sensor_handles The list of sensor handles on which user is detected.
 * @param[in] user_data  The user data passed in ua_monitor_set_user_presence_detected_cb()
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_set_user_presence_detected_cb()
 */
typedef void (*ua_presence_user_detected_cb)(
		int result,
		ua_monitor_h handle,
		ua_user_h user_handle,
		GSList *device_handles,
		GSList *sensor_handles,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to get all registered devices one by one.
 * @since_tizen 5.5
 *
 * @remarks The @a device_handle should not be released.
 * @remarks The @a device_handle can be used only in the callback.
 *
 * @param[in] device_handle The device handle.
 * @param[in] user_data User data passed in ua_device_foreach_added().
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_foreach_added()
 */
typedef bool (*ua_registered_dev_cb)(
		ua_device_h device_handle,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to get all user handles.
 * @since_tizen 5.5
 *
 * @remarks The @a user_handle should not be released.
 * @remarks The @a user_handle can be used only in the callback.
 *
 * @param[in] user_handle The user handle.
 * @param[in] user_data User data passed in ua_user_foreach_added().
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_foreach_added()
 */
typedef bool (*ua_registered_user_cb)(
		ua_user_h user_handle,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to get all service handles.
 * @since_tizen 5.5
 *
 * @remarks The @a service_handle should not be released.
 * @remarks The @a service_handle can be used only in the callback.
 *
 * @param[in] service_handle The service handle.
 * @param[in] user_data User data passed in ua_service_foreach_added_services().
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_foreach_added_services()
 */
typedef bool (*ua_service_added_service_cb)(
		ua_service_h service_handle,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to get the user handle added to the service.
 * @since_tizen 5.5
 *
 * @remarks The @a service_handle should not be released.
 * @remarks The @a service_handle can be used only in the callback.
 * @remarks The @a user_handle should not be released.
 * @remarks The @a user_handle can be used only in the callback.
 *
 * @param[in] service_handle The service handle.
 * @param[in] user_handle The user handle.
 * @param[in] user_data User data passed in ua_service_foreach_added_users().
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_foreach_added_users()
 */
typedef bool (*ua_service_added_user_cb)(
		ua_service_h service_handle,
		ua_user_h user_handle,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to get the device handle added to the service.
 * @since_tizen 5.5
 *
 * @remarks The @a service_handle should not be released.
 * @remarks The @a service_handle can be used only in the callback.
 * @remarks The @a device_handle should not be released.
 * @remarks The @a device_handle can be used only in the callback.
 *
 * @param[in] service_handle The service handle.
 * @param[in] device_handle The device handle.
 * @param[in] user_data User data passed in ua_service_foreach_added_devices().
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_foreach_added_devices()
 */
typedef bool (*ua_service_added_device_cb)(
		ua_service_h service_handle,
		ua_device_h device_handle,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Initializes an user awareness framework.
 * @since_tizen 5.5
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 * @retval #UA_ERROR_ALREADY_DONE Already initialized
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_deinitialize()
 */
int ua_initialize(void);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief De-initializes an user awareness framework.
 * @since_tizen 5.5
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_NOT_INITIALIZED Not initialized yet
 * @retval #UA_ERROR_ALREADY_DONE Already initialized
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_deinitialize()
 */
int ua_deinitialize(void);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Registers the application for wakeup before dbus event.
 * @since_tizen 5.5
 *
 * @param[in] app_id Application id
 * @param[in] uid User id
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOT_PERMITTED Operation not permitted
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_unset_app_info()
 */
int ua_set_app_info(const char *app_id, unsigned short uid);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Unregisters an application, if it no more require wakeup to receive dbus events.
 * @since_tizen 5.5
 *
 * @param[in] app_id Application id
 * @param[in] uid User id
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOT_PERMITTED Operation not permitted
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_set_app_info()
 */
int ua_unset_app_info(const char *app_id, unsigned short uid);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Creates the ua monitor's handle.
 * @since_tizen 5.5
 *
 * @remarks The @a handle should be destroyed by using #ua_monitor_destroy().
 *
 * @param[out] handle The monitor handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_destroy()
 */
int ua_monitor_create(
		ua_monitor_h *handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Destroys the ua monitor's handle.
 * @since_tizen 5.5
 *
 * @param[in] handle The monitor handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_create()
 */
int ua_monitor_destroy(
		ua_monitor_h handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Adds sensors to UA monitor for which monitoring is to be done.
 * @since_tizen 5.5
 *
 * @param[in] handle The monitor handle
 * @param[in] sensor The sensor to be added to monitor
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOT_PERMITTED Operation not permitted
 * @retval #UA_ERROR_ALREADY_DONE Already done
 * @retval #UA_ERROR_NOT_READY Resource not ready
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_remove_sensor()
 * @see ua_monitor_foreach_sensor()
 */
int ua_monitor_add_sensor(
		ua_monitor_h handle,
		ua_sensor_e sensor);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Removes sensors from UA monitor.
 * @since_tizen 5.5
 *
 * @param[in] handle The monitor handle
 * @param[in] sensor Sensor to be removed from monitor
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOT_PERMITTED Operation not permitted
 * @retval #UA_ERROR_ALREADY_DONE Already done
 * @retval #UA_ERROR_NOT_READY Resource not ready
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_add_sensor()
 * @see ua_monitor_foreach_sensor()
 */
int ua_monitor_remove_sensor(
		ua_monitor_h handle,
		ua_sensor_e sensor);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Retrieves all sensors present in UA monitor.
 * @since_tizen 5.5
 *
 * @param[in] handle The monitor handle
 * @param[in] foreach_cb Callback function to be invoked foreach sensor present in monitor handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_add_sensor()
 * @see ua_monitor_remove_sensor()
 */
int ua_monitor_foreach_sensor(
		ua_monitor_h handle,
		ua_monitor_foreach_sensor_cb foreach_cb,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Checks if sensor is available or not.
 * @since_tizen 5.5
 *
 * @param[in] sensor The sensor for which status to be checked.
 * @param[out] available TRUE if sensor is available FALSE otherwise.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 */
int ua_monitor_is_sensor_available(
		ua_sensor_e sensor,
		bool *available);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets sensor state changed callback.
 * @since_tizen 5.5
 *
 * @param[in] handle The monitor handle
 * @param[in] callback Sensor state changed callback
 * @param[in] user_data The user data to be passed in sensor state changed callback.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_unset_sensor_state_cb()
 */
int ua_monitor_set_sensor_state_cb(
		ua_monitor_h handle,
		ua_monitor_sensor_state_changed_cb callback,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Unsets sensor state changed callback.
 * @since_tizen 5.5
 *
 * @param[in] handle The monitor handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_set_sensor_state_cb()
 */
int ua_monitor_unset_sensor_state_cb(
		ua_monitor_h handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets absence detected callback for all users.
 * @since_tizen 5.5
 *
 * @param[in] handle The monitor handle
 * @param[in] callback Absence detected callback
 * @param[in] user_data The user data
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_unset_user_absence_detected_cb()
 */
int ua_monitor_set_user_absence_detected_cb(
		ua_monitor_h handle,
		ua_absence_user_detected_cb callback,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets presence detected callback for all users.
 * @since_tizen 5.5
 *
 * @param[in] handle The monitor handle
 * @param[in] callback Presence detected callback
 * @param[in] user_data The user data
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_unset_user_presence_detected_cb()
 */
int ua_monitor_set_user_presence_detected_cb(
		ua_monitor_h handle,
		ua_presence_user_detected_cb callback,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Unsets absence detected callback for all users.
 * @since_tizen 5.5
 *
 * @param[in] handle The monitor handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_set_user_absence_detected_cb()
 */
int ua_monitor_unset_user_absence_detected_cb(
		ua_monitor_h handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Unsets presence detected callback for all users.
 * @since_tizen 5.5
 *
 * @param[in] handle The monitor handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_set_user_presence_detected_cb()
 */
int ua_monitor_unset_user_presence_detected_cb(
		ua_monitor_h handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets user presence detected condition to UA monitor.
 * @since_tizen 5.5
 *
 * @remarks bitmask_and and bitmask_or must contain different set of sensors.
 *
 * @param[in] handle The monitor handle.
 * @param[in] bitmask_and Bitmask for sensors which will be AND operated for checking User presence.
 * @param[in] bitmask_or Bitmask for sensors which will be OR operated for checking User presence.
 * @param[in] conjunction_op Logical operator in between AND and OR user presence condition.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_condition_conjunction_e
 */
int ua_monitor_set_user_presence_condition(
		ua_monitor_h handle,
		unsigned int bitmask_and,
		unsigned int bitmask_or,
		ua_condition_conjunction_e conjunction_op);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets user absence detected condition to UA monitor.
 * @since_tizen 5.5
 *
 * @remarks bitmask_and and bitmask_or must contain different set of sensors.
 *
 * @param[in] handle The monitor handle.
 * @param[in] bitmask_and Bitmask for sensors which will be AND operated for checking User absence.
 * @param[in] bitmask_or Bitmask for sensors which will be OR operated for checking User absence.
 * @param[in] conjunction_op Logical operator in between AND and OR user absence condition.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_condition_conjunction_e
 */
int ua_monitor_set_user_absence_condition(
		ua_monitor_h handle,
		unsigned int bitmask_and,
		unsigned int bitmask_or,
		ua_condition_conjunction_e conjunction_op);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Starts scan of registered devices.
 * @since_tizen 5.5
 *
 * @remarks Only one scan can run at one time.
 *
 * @param[in] handle The monitor handle
 * @param[in] scan_time_multiplier A value which we will multiply by UA_SCAN_TIME_MULTIPLIER.
 *            This is maximum time for which scan can run, but scan can complete earlier also.
 * @param[in] callback It will be called for each scanned device one by one, and on scan
 *            completion too.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOW_IN_PROGRESS Operation now in progress
 * @retval #UA_ERROR_NO_DATA No sensor available
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see UA_SCAN_TIME_MULTIPLIER
 * @see ua_scan_completed_cb()
 * @see ua_monitor_cancel_scan_devices()
 */
int ua_monitor_start_scan_devices(
		ua_monitor_h handle,
		int scan_time_multiplier,
		ua_scan_completed_cb callback,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Stops ua_monitor_start_scan_devices().
 * @since_tizen 5.5
 *
 * @param[in] monitor  UA monitor's handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOT_IN_PROGRESS Operation not in progress
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_start_scan_devices()
 */
int ua_monitor_cancel_scan_devices(ua_monitor_h monitor);


/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Starts user presence detection.
 * @since_tizen 5.5
 *
 * @remarks This function is not specific for any particular user's presence detection.
 * Therefore, the callback will be invoked when any user is detected by sensors.
 *
 * @param[in] handle The monitor handle
 * @param[in] service_handle The service handle. If service is NULL then montior will start for default service.
 * @param[in] mode  User detection mode. \n
 *  #UA_DETECT_MODE_ALL_SENSOR : Detection callback will be invoked only after a user is
 *  detected by all sensors added to monitor. \n
 *  #UA_DETECT_MODE_ANY_SENSOR : Detection callback will be invoked as soon as a user is
 *  detected by any one of the sensors. \n
 * @param[in] callback Callback to be invoked after device presence detection.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOW_IN_PROGRESS Operation now in progress
 * @retval #UA_ERROR_NO_DATA No sensor available
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_presence_detected_cb()
 * @see ua_monitor_stop_presence_detection()
 */
int ua_monitor_start_presence_detection(
		ua_monitor_h handle,
		ua_service_h service_handle,
		ua_detection_mode_e mode,
		ua_presence_detected_cb callback,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Stops user presence detection.
 * @since_tizen 5.5
 *
 * @param[in] monitor  UA monitor's handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOT_IN_PROGRESS Operation not in progress
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_start_presence_detection()
 */
int ua_monitor_stop_presence_detection(
		ua_monitor_h monitor);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Starts user absence detection.
 * @since_tizen 5.5
 *
 * @remarks This function is not specific for any particular user's absence detection.
 * Therefore, the callback will be invoked only when no user is detected by sensors.
 *
 * @param[in] handle The monitor handle
 * @param[in] service_handle The service handle. If service is NULL then montior will start for default service.
 * @param[in] mode User detection mode. \n
 *  #UA_DETECT_MODE_ALL_SENSOR : Detection callback will be invoked only after no user is
 *  detected by all sensors added to monitor. \n
 *  #UA_DETECT_MODE_ANY_SENSOR : Detection callback will be invoked as soon as no user is
 *  detected by any one of the sensors. \n
 * @param[in] callback  Callback to be invoked after absence detection.
 * @param[in] user_data  The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOW_IN_PROGRESS Operation now in progress
 * @retval #UA_ERROR_NO_DATA No sensor available
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_absence_detected_cb()
 * @see ua_monitor_stop_absence_detection()
 */
int ua_monitor_start_absence_detection(
		ua_monitor_h handle,
		ua_service_h service_handle,
		ua_detection_mode_e mode,
		ua_absence_detected_cb callback,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Stops user absence detection.
 * @since_tizen 5.5
 *
 * @param[in] handle The monitor handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOT_IN_PROGRESS Operation not in progress
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_monitor_start_absence_detection()
 */
int ua_monitor_stop_absence_detection(
		ua_monitor_h handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Creates the user handle.
 * @since_tizen 5.5
 *
 * @remarks The @a user_handle should be destroyed by using #ua_user_destroy().
 *
 * @param[in] user_handle The user handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_destroy()
 */
int ua_user_create(
		ua_user_h *user_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Destroys the user handle.
 * @since_tizen 5.5
 *
 * @remarks User context will removed and all allocated memory (if any) will be released.
 *
 * @param[in] user_handle The user handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_create()
 */
int ua_user_destroy(
		ua_user_h user_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Adds the user.
 * @since_tizen 5.5
 *
 * @param[in] user_handle The user handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_remove()
 */
int ua_user_add(
		ua_user_h user_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Removes the user.
 * @since_tizen 5.5
 *
 * @param[in] user_handle The user handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_add()
 */
int ua_user_remove(
		ua_user_h user_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets default user handle.
 * @since_tizen 5.5
 *
 *TODO lk, shall we destroy??
 * @remarks The @a user_handle should be released using #ua_user_destroy().
 *
 * @param[out] user_handle The user handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_get_account()
 * @see ua_user_get_name()
 */
int ua_user_get_default_user(
		ua_user_h * user_handle);
/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets account info for device handle.
 * @since_tizen 5.5
 *
 * @param[in] user_handle The user handle
 * @param[in] account Account information
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_get_account()
 */
int ua_user_set_account(
		ua_user_h user_handle,
		const char *account);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets account info for user handle.
 * @since_tizen 5.5
 *
 * @remarks You must release @a account using g_free().
 * @param[in] user_handle The user handle
 * @param[out] account  Account information
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_get_account()
 */
int ua_user_get_account(
		ua_user_h user_handle,
		char **account);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets name info for user handle.
 * @since_tizen 5.5
 *
 * @param[in] user_handle The user handle
 * @param[in] name The user name information
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_get_name()
 */
int ua_user_set_name(
		ua_user_h user_handle,
		const char *name);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets name info for user handle.
 * @since_tizen 5.5
 *
 * @remarks You must release @a name using g_free().
 *
 * @param[in] user_handle The user handle
 * @param[out] name The user name information
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_set_name()
 */
int ua_user_get_name(
		ua_user_h user_handle,
		char **name);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets last presence timestamp info for user handle.
 * @since_tizen 5.5
 *
 * @param[in] user_handle The user handle.
 * @param[out] timestamp The last presence timestamp information for UA user.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 */
int ua_user_get_last_presence(ua_user_h user_handle, long *timestamp);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets user handle by account.
 * @since_tizen 5.5
 *
 * @remarks The @a user_handle should not be released.
 * @remarks The @a user_handle can be used only in the fuction.
 *
 * @param[in] account The user account information
 * @param[out] user_handle The user handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 */
int ua_user_get_handle_by_account(const char* account, ua_user_h *user_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Callback to be invoked if a new device added to UA.
 * @since_tizen 5.5
 *
 * @remarks The @a dev_handle should not be released.
 * @remarks The @a dev_handle can be used only in the callback.
 *
 * @param[in] result The result of the requested operation.
 * @param[in] dev_handle The added device's handle
 * @param[in] user_data The user data passed in ua_user_add_device()
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_add_device()
 */
typedef void (*ua_user_device_added_cb)(
		int result,
		ua_device_h dev_handle,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Adds device for a user.
 * @since_tizen 5.5
 *
 * @param[in] user_handle The user handle.
 * @param[in] device_handle The device handle.
 * @param[in] callback Callback to be invoked after adding the user done.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_remove_device()
 * @see ua_user_device_added_cb()
 */
int ua_user_add_device(
		ua_user_h user_handle,
		ua_device_h device_handle,
		ua_user_device_added_cb callback,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Removes the device handle for a user handle.
 * @since_tizen 5.5
 *
 * @param[in] user_handle The user handle.
 * @param[in] device_handle The device handle.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retcal #UA_ERROR_RESOURCE_BUSY Remove done. but another service uses thie device.
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_add_device()
 */
int ua_user_remove_device(
		ua_user_h user_handle,
		ua_device_h device_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Retrieves the user handle of all the registered users.
 * @since_tizen 5.5
 *
 * @param[in] foreach_cb Callback function to be invoked with user handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_registered_user_cb()
 */
int ua_user_foreach_added(
		ua_registered_user_cb foreach_cb,
		void *user_data);


/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Creates the service handle.
 * @since_tizen 5.5
 *
 * @remarks The @a service_handle should be destroyed by using #ua_service_destroy().
 *
 * @param[in] service_handle The service handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_destroy()
 */
int ua_service_create(
		ua_service_h *service_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Destroys the service handle.
 * @since_tizen 5.5
 *
 * @remarks Service context will removed and all allocated memory (if any) will be released.
 *
 * @param[in] service_handle The service handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_create()
 */
int ua_service_destroy(
		ua_service_h service_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Adds the service.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_remove()
 */
int ua_service_add(
		ua_service_h service_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Removes the service.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_add()
 */
int ua_service_remove(
		ua_service_h service_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets default service handle.
 * @since_tizen 5.5
 *
 * @remarks The @a service_handle is managed by the platform and will be released
 *             when #ua_deinitialize() is called.
 *
 * @param[out] service_handle The service handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_get_name()
 * @see ua_service_set_name()
 */
int ua_service_get_default_service(
		ua_service_h * service_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets service handle by service name.
 * @since_tizen 5.5
 *
 * @remarks The @a service_handle should be destroyed by using #ua_service_destroy().
 *
 * @param[in] name The service name information
 * @param[out] service_handle The service handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_get_name()
 * @see ua_service_set_name()
 */
int ua_service_get_by_name(
		const char *name,
		ua_service_h *service_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets detection threshold for service handle.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle
 * @param[in] presence_threshold The service presence threshold information
 * @param[in] absence_threshold The service absence threshold information
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 */
int ua_service_set_detection_threshold(
		ua_service_h service_handle,
		unsigned int presence_threshold,
		unsigned int absence_threshold);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets name info for service handle.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle
 * @param[in] name The service name information
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_get_name()
 */
int ua_service_set_name(
		ua_service_h service_handle,
		const char *name);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets name info for service handle.
 * @since_tizen 5.5
 *
 * @remarks You must release @a name using g_free().
 *
 * @param[in] service_handle The service handle
 * @param[out] name The service name information
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_set_name()
 */
int ua_service_get_name(
		ua_service_h service_handle,
		char **name);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Adds a user for a specific service.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle.
 * @param[in] user_handle The user handle.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_remove_user()
 */
int ua_service_add_user(
		ua_service_h service_handle,
		ua_user_h user_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Removes the user for the specific service handle.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle.
 * @param[in] user_handle The user handle.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_add_user()
 */
int ua_service_remove_user(
		ua_service_h service_handle,
		ua_user_h user_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Adds device for a specific service.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle.
 * @param[in] device_handle The device handle.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_remove_device()
 */
int ua_service_add_device(
		ua_service_h service_handle,
		ua_device_h device_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Removes the device from a specific service handle.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle.
 * @param[in] device_handle The device handle.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_add_device()
 */
int ua_service_remove_device(
		ua_service_h service_handle,
		ua_device_h device_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets device discriminant for a specific service.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle.
 * @param[in] device_handle The device handle.
 * @param[in] discriminant The device discriminant.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_add_device()
 * @see ua_service_remove_device()
 */
int ua_service_set_device_discriminant(
		ua_service_h service_handle,
		ua_device_h device_handle,
		gboolean discriminant);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device discriminant for a specific service.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle.
 * @param[in] device_handle The device handle.
 * @param[out] discriminant The device discriminant.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_add_device()
 * @see ua_service_remove_device()
 */
int ua_service_get_device_discriminant(
		ua_service_h service_handle,
		ua_device_h device_handle,
		gboolean *discriminant);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Retrieves the service handle of all the added services.
 * @since_tizen 5.5
 *
 * @param[in] foreach_cb Callback function to be invoked with service handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_added_service_cb()
 */
int ua_service_foreach_added_services(
		ua_service_added_service_cb foreach_cb,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Retrieves the user handle of all the added users for a specific service.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle.
 * @param[in] foreach_cb Callback function to be invoked with user handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_added_user_cb()
 */
int ua_service_foreach_added_users(
		ua_service_h service_handle,
		ua_service_added_user_cb foreach_cb,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Retrieves the device handle of all the added devices for a specific service.
 * @since_tizen 5.5
 *
 * @param[in] service_handle The service handle.
 * @param[in] foreach_cb Callback function to be invoked with device handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_added_device_cb()
 */
int ua_service_foreach_added_devices(
		ua_service_h service_handle,
		ua_service_added_device_cb foreach_cb,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Creates the device handle.
 * @since_tizen 5.5
 *
 * @remarks The @a device_handle should be destroyed by using #ua_user_destroy().
 *
 * @param[out] device_handle The device handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_destroy()
 */
int ua_device_create(
		ua_device_h * device_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Clones device handle if application wants to store it after foreach device
 *        handles. It should be call before foreach callback return.
 * @since_tizen 5.5
 *
 * @remarks The @a cloned should be released using #ua_device_destroy().
 *
 * @param[out] cloned The cloned device handle.
 * @param[in] origin The input device handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_registered_dev_cb()
 * @see ua_device_destroy()
 */
int ua_device_clone(
		ua_device_h *cloned,
		ua_device_h origin);


/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Destroys the device handle.
 * @since_tizen 5.5
 *
 * @remarks Device context will removed and all allocated memory (if any) will be released.
 *
 * @param[in] device_handle The device handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_create()
 */
int ua_device_destroy(
		ua_device_h device_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets a device type info for a device.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[in] mac_type The MAC address type of the device
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_mac_type()
 */
int ua_device_set_mac_type(
		ua_device_h device_handle,
		ua_mac_type_e mac_type);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device type info from the device.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[out] mac_type The MAC address type of the device
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_set_mac_type()
 */
int ua_device_get_mac_type(
		ua_device_h device_handle,
		ua_mac_type_e *mac_type);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets device's operating system info.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[in] os The device's operating system.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_os_info()
 */
int ua_device_set_os_info(
		ua_device_h device_handle,
		ua_os_type_e os);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device's operating system info.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[out] os_info Device's operating system info.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_os_info()
 */
int ua_device_get_os_info(
		ua_device_h device_handle,
		ua_os_type_e *os_info);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets device's MAC address.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[in] mac_address The device's MAC address.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_mac_address()
 */
int ua_device_set_mac_address(
		ua_device_h device_handle,
		const char *mac_address);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device's MAC address.
 * @since_tizen 5.5
 *
 * @remarks You must release @a mac_address using g_free().
 *
 * @param[in] device_handle The device handle
 * @param[out] mac_address The device's MAC address.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_set_mac_address()
 */
int ua_device_get_mac_address(
		ua_device_h device_handle,
		char **mac_address);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets device ID.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[in] device_id The device ID.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_device_id()
 */
int ua_device_set_device_id(
		ua_device_h device_handle,
		const char *device_id);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device ID.
 * @since_tizen 5.5
 *
 * @remarks You must release @a device_id using g_free().
 * @param[in] device_handle The device handle
 * @param[out] device_id The device ID.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @see ua_device_set_device_id()
 */
int ua_device_get_device_id(
		ua_device_h device_handle,
		char **device_id);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets device's Wi-Fi BSSID.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[in] bssid  The device's Wi-Fi BSSID.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_wifi_bssid()
 */
int ua_device_set_wifi_bssid(
		ua_device_h device_handle,
		const char *bssid);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device's Wi-Fi BSSID.
 * @since_tizen 5.5
 *
 * @remarks You must release @a bssid using g_free().
 * @param[in] device_handle The device handle
 * @param[out] bssid The device's Wi-Fi BSSID.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_set_wifi_bssid()
 */
int ua_device_get_wifi_bssid(
		ua_device_h device_handle,
		char **bssid);
/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets device's Wi-Fi IPv4 address.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[in] ipv4_address The device's wifi IPv4 address.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_wifi_ipv4_address()
 */
int ua_device_set_wifi_ipv4_address(
		ua_device_h device_handle,
		const char *ipv4_address);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device's Wi-Fi IPv4 address.
 * @since_tizen 5.5
 *
 * @remarks You must release @a ipv4_address using g_free().
 *
 * @param[in] device_handle The device handle
 * @param[out] ipv4_address The device's Wi-Fi IPv4 address.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_set_wifi_ipv4_address()
 */
int ua_device_get_wifi_ipv4_address(
		ua_device_h device_handle,
		char **ipv4_address);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets last presence timestamp info for device handle.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[out] timestamp The last presence timestamp information for UA device
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 */
int ua_device_get_last_presence(
		ua_device_h device_handle,
		long *timestamp);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets whether pairing is required for the user device.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[out] pairing_required TRUE if pairing is required, FALSE otherwise.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 */
int ua_device_get_pairing_required(
		ua_device_h device_handle,
		bool *pairing_required);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets discriminant for a device.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[in] discriminant Determines whether to judge PRESENCE/ABSENCE.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_discriminant()
 */

int ua_device_set_discriminant(
		ua_device_h device_handle,
		bool discriminant);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets discriminant from the device.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 * @param[out] discriminant Determines whether to judge PRESENCE/ABSENCE.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_set_discriminant()
 */
int ua_device_get_discriminant(
		ua_device_h device_handle,
		bool *discriminant);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Updates the device.
 * @since_tizen 5.5
 *
 * @param[in] device_handle The device handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OPERATION_FAILED Operation failed
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_set_os_info()
 * @see ua_device_set_wifi_ipv4_address()
 * @see ua_device_set_discriminant()
 * @see ua_device_set_payload_service_id()
 * @see ua_device_set_payload_device_icon()
 * @see ua_device_set_payload_purpose()
 * @see ua_device_set_payload_duid()
 */
int ua_device_update(
		ua_device_h device_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets BLE payload service_id for device.
 * @since_tizen 5.5
 *
 * @param[in] handle The device handle.
 * @param[in] service_id The BLE payload service_id.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_payload_service_id()
 */
int ua_device_set_payload_service_id(
		ua_device_h handle,
		const char service_id);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets BLE payload device_icon for device.
 * @since_tizen 5.5
 *
 * @param[in] handle The device handle.
 * @param[in] device_icon The BLE payload device_icon.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_payload_device_icon()
 */
int ua_device_set_payload_device_icon(
		ua_device_h handle,
		const char device_icon);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets BLE payload purpose for device.
 * @since_tizen 5.5
 *
 * @param[in] handle The device handle.
 * @param[in] purpose The BLE payload purpose.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_payload_purpose()
 */
int ua_device_set_payload_purpose(
		ua_device_h handle,
		const char purpose);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets BLE payload DUID for device.
 * @since_tizen 5.5
 *
 * @remarks The @a duid is expected to be exact 17 bytes long data
 *          and may have intermediate NULLs
 *
 * @param[in] handle The device handle.
 * @param[in] duid_len The BLE payload DUID's length.
 * @param[in] duid The BLE payload DUID.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_get_payload_duid()
 */
int ua_device_set_payload_duid(
		ua_device_h handle,
		unsigned int duid_len,
		const char *duid);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sets iBeacon adevertising data.
 * @since_tizen 5.5
 *
 * @param[in] adv_len The iBeacon adevertising data's length.
 * @param[in] ibeacon_adv The iBeacon adevertising data.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 */
int ua_add_ibeacon_adv_data(
		unsigned int adv_len,
		const char *ibeacon_adv);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device's BLE payload service_id.
 * @since_tizen 5.5
 *
 * @param[in] handle The device handle.
 * @param[out] service_id The BLE payload service_id.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_set_payload_service_id()
 */
int ua_device_get_payload_service_id(
		ua_device_h handle,
		char *service_id);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device's BLE payload device_icon.
 * @since_tizen 5.5
 *
 * @param[in] handle The device handle.
 * @param[out] device_icon The BLE payload device_icon.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_set_payload_device_icon()
 */
int ua_device_get_payload_device_icon(
		ua_device_h handle,
		char *device_icon);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device's BLE payload purpose.
 * @since_tizen 5.5
 *
 * @param[in] handle The device handle.
 * @param[out] purpose The BLE payload purpose.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_set_payload_purpose()
 */
int ua_device_get_payload_purpose(
		ua_device_h handle,
		char *purpose);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device's BLE payload DUID.
 * @since_tizen 5.5
 *
 * @remarks You must release @a duid using g_free().
 *
 * @param[in] handle The device handle.
 * @param[out] duid The BLE payload DUID.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_device_set_payload_duid()
 */
int ua_device_get_payload_duid(
		ua_device_h handle,
		char **duid);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device handle by MAC address.
 * @since_tizen 5.5
 *
 * @remarks The @a device_handle should not be released.
 * @remarks The @a device_handle can be used only in the function.
 *
 * @param[in] mac The device's MAC address
 * @param[out] device_handle The device handle
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 *
 */
int ua_device_get_by_mac_address(
		const char *mac,
		ua_device_h *device_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets device handle by device ID.
 * @since_tizen 5.5
 *
 * @remarks The @a device_handle should not be released.
 * @remarks The @a device_handle can be used only in the function.
 *
 * @param[in] device_id The device ID
 * @param[in] mac_type The device's MAC type
 * @param[out] device_handle The device handle.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 *
 */
int ua_device_get_by_device_id(
		const char *device_id,
		ua_mac_type_e mac_type,
		ua_device_h * device_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Removes the device by MAC address.
 * @since_tizen 5.5
 *
 * @param[in] mac The device's MAC address.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retcal #UA_ERROR_RESOURCE_BUSY Remove done. but another service uses thie device.
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_remove_device_by_device_id()
 */
int ua_user_remove_device_by_mac_address(
		const char *mac);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Removes the device ID.
 * @since_tizen 5.5
 *
 * @param[in] device_id The device ID.
 * @param[in] mac_type The device's MAC address type.
 *
 * @return  0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retcal #UA_ERROR_RESOURCE_BUSY Remove done. but another service uses thie device.
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_user_remove_device_by_mac_address()
 */
int ua_user_remove_device_by_device_id(
		const char *device_id,
		ua_mac_type_e mac_type);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Retrieves the device handle of all the registered devices.
 * @since_tizen 5.5
 *
 * @param[in] foreach_cb Callback function to be invoked with all device handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_registered_dev_cb()
 */
int ua_device_foreach_added(
		ua_registered_dev_cb foreach_cb,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Retrieves the device handle of all the registered devices for one specific user.
 * @since_tizen 5.5
 *
 * @param[in] user_handle The user handle.
 * @param[in] foreach_cb Callback function to be invoked with user device handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_registered_dev_cb()
 */
int ua_device_foreach_added_by_user(
		ua_user_h user_handle,
		ua_registered_dev_cb foreach_cb,
		void *user_data);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets sensor's timestamp.
 * @since_tizen 5.5
 *
 * @param[in] sensor_handle The sensor handle
 * @param[out] timestamp The sensor's timestamp.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 */
int ua_sensor_get_timestamp(
		ua_sensor_h sensor_handle,
		long int *timestamp);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets sensor's timestamp.
 * @since_tizen 5.5
 *
 * @param[in] sensor_handle The sensor handle
 * @param[out] bitmask The sensor's bitmask.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 */
int ua_sensor_get_bitmask(
		ua_sensor_h sensor_handle,
		ua_sensor_e *bitmask);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets sensor's information values count.
 * @since_tizen 5.5
 *
 * @param[in] sensor_handle The sensor handle
 * @param[out] info_count The sensor's information count.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 */
int ua_sensor_get_info_count(
		ua_sensor_h sensor_handle,
		int *info_count);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets sensor's information values.
 * @since_tizen 5.5
 *
 * @param[in] sensor_handle The sensor handle
 * @param[out] info_values The sensor's information values.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 */
GSList *ua_sensor_get_info_values(
		ua_sensor_h sensor_handle);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif
#endif /* __TIZEN_CAPI_NETWORK_USER_AWARENESS_H__ */
